warning off

// Endogenous variables
var welfhh Util1 Util2
    y yw c k inv h hd f n phi b bcb re lamn lam r mc pinf pstar x1 x2 
    vp wh w wstar g1 g2 vw pk xi q rl rlong qeh rlongeh tp real epinf 
    bal2y
    yf ywf cf kf invf hf hdf ff nf phif bf bcbf ref lamnf lamf rf mcf pinff pstarf
    vpf whf wf wstarf vwf pkf xif qf rlf bal2yf;

// Exogenous variables
var epsa epsf epsi epsn;

varexo ea en ef ei;

parameters  BETA ZETA GAMMA PSI ETA ALPHA DELTA KAPPA EPSP EPSW PINFSS RESS
            PHIP PHIW PHII PHIN RHOR UPSIPI UPSIY NSS BG TPSS
            RHOA SIGA RHON SIGP RHOF SIGF EPSF RHOI SIGI SIGR YSS
            RHORE1 RHORE2 SIGRE SK SW SP RLONGSS RHOB NUTP NUSLOPE UPSIN UPSITP UPSISLOPE;

model;
// OPTIMAL MONETARY POLICY
// Policy Rule, agent 1
    log(r) = RHOR*log(r(-1)) 
    + (1-RHOR)*log(PINFSS/BETA) 
    + (1-RHOR)*UPSIPI*log(pinf/PINFSS) 
    + (1-RHOR)*UPSIY*log(y/yf)
    + (1-RHOR)*UPSIN*log(pinf/PINFSS*y/y(-1))
    + (1-RHOR)*UPSITP*(tp - TPSS)
    + (1-RHOR)*UPSISLOPE*(rlong - r - TPSS);

// Policy Rule, agent 2
    bal2y = RHOB*bal2y(-1) + (1-RHOB)*(RESS/YSS + NUTP*(tp - TPSS) + NUSLOPE*(rlong - r - TPSS));
// 3  Utility 1: lmult3
    Util1 = epsn*(c^(1-GAMMA)/(1-GAMMA) - PSI*h^(1+ETA)/(1+ETA));
// 4  Utility 2: lmult4
    Util2 = epsn*(c^(1-GAMMA)/(1-GAMMA) - PSI*h^(1+ETA)/(1+ETA));

// MODEL
// 5  Ex ante real rate
    real = r/pinf(+1);
// 6  Consumption Euler equation
    1 = lamn(+1)*r;
// 7  Labor supply
    wh = PSI*h^ETA*c^GAMMA;
// 8  Nominal SDF
    lamn = BETA*epsn/epsn(-1)*(c(-1)/c)^GAMMA/pinf;
// 9  Real SDF
    lam = lamn*pinf;
// 10 Reset wage
    wstar = (1+SW)*EPSW/(EPSW-1)*g1/g2;
// 11 Wage numerator
    g1 = wh*w^EPSW*hd+PHIW*lam(+1)*pinf(+1)^EPSW*g1(+1);
// 12 Wage denominator
    g2 = w^EPSW*hd+PHIW*lam(+1)*pinf(+1)^(EPSW-1)*g2(+1);
// 13 Labor market clearing
    h = hd*vw;
// 14 Wage dispersion
    vw = (1-PHIW)*(wstar/w)^(-EPSW)+PHIW*pinf^EPSW*(w/w(-1))^EPSW*vw(-1);
// 15 Aggregate wage
    w^(1-EPSW)=(1-PHIW)*wstar^(1-EPSW)+PHIW*pinf^(EPSW-1)*w(-1)^(1-EPSW);
// 16 Labor demand
    w = (1+SP)*mc*(1-ALPHA)*yw/hd;
// 17 Capital return
    pk*xi = lam(+1)*((1+SK)*mc(+1)*ALPHA*yw(+1)/k+(1-DELTA)*pk(+1)*xi(+1));
// 18 Optimal investment
     1 = pk*epsi*(1 - PHII*(inv/inv(-1)-1)*inv/inv(-1) - PHII/2*(inv/inv(-1)-1)^2)
        + lam(+1)*xi(+1)/xi*pk(+1)*epsi(+1)*PHII*(inv(+1)/inv-1)*(inv(+1)/inv)^2;
// 19 Bond pricing
    q*xi = lamn(+1)*(1+KAPPA*q(+1)*xi(+1));
// 20 Capital accumulation equation
    k = (1-DELTA)*k(-1) + epsi*(1-PHII/2*(inv/inv(-1)-1)^2)*inv;
// 21 Loan-in-advance constraint
    inv = q*(f-KAPPA*f(-1)/pinf);
// 22 Reset price
    pstar = EPSP/(EPSP-1)*x1/x2;
// 23 Price numerator
    x1 = mc*y+PHIP*lam(+1)*pinf(+1)^EPSP*x1(+1);
// 24 Price denominator
    x2 = y+PHIP*lam(+1)*pinf(+1)^(EPSP-1)*x2(+1);
// 25 Aggregate production
    yw = epsa*k(-1)^ALPHA*hd^(1-ALPHA);
// 26 Goods market clearing
    yw = y*vp;
// 27 Price dispersion
    vp = (1-PHIP)*pstar^(-EPSP)+PHIP*pinf^EPSP*vp(-1);
// 28 Inflation
    1 = (1-PHIP)*pstar^(1-EPSP)+PHIP*pinf^(EPSP-1);
// 29 Long-term rate
    rl = (1+KAPPA*q)/q(-1);
// 30 Equity adjustment
    1+PHIN*n/NSS*(n/NSS-1)+PHIN/2*(n/NSS-1)^2 = ZETA*lamn(+1)*(r+(rl(+1)-r)*phi);
// 31 Risk-weighted leverage constraint
    epsf*phi*lamn(+1)*rl(+1) = lamn(+1)*(r+(rl(+1)-r)*phi);
// 32 Risk-weighted leverage
    phi*n = q*(f+BG)-re;
// 33 Central bank balance sheet
    q*bcb = re;
// 34 Debt-market clearing
    b = BG - bcb;
// 35 Aggregate resource constraint
    y = c + inv;
// 36 Long-term yield
    rlong = 1/q+KAPPA;
// 37 Expectations hypothesis bond price
    qeh = (1+KAPPA*qeh(+1))/r;
// 38 Expectations hypothesis long-term yield
    rlongeh = 1/qeh+KAPPA;
// 39 Term premium
    tp = rlong - rlongeh;

// WELFARE MEASURE
// 40 Household welfare
    welfhh = Util1 + BETA*welfhh(+1);

// 41 Expected inflation
    epinf = pinf(+1);
// 42 Relative size of the balance sheet
    bal2y = re/y;
// 43 Lag relative size of the balance sheet
//    bal2yl = bal2y(-1);

// EXOGENOUS PROCESSES
// 44 Technology
    log(epsa) = RHOA*log(epsa(-1)) + SIGA*ea;
// 45 Preferences
    log(epsn) = RHON*log(epsn(-1)) - SIGP*en;
// 46 Financial
    log(epsf) = (1-RHOF)*log(EPSF) + RHOF*log(epsf(-1)) - SIGF*ef;
// 47 Marginal efficiency of investment
    log(epsi) = RHOI*log(epsi(-1)) + SIGI*ei;

// FLEXIBLE ECONOMY
// 48 Interest rate policy
    log(rf) = RHOR*log(rf(-1)) + (1-RHOR)*(log(PINFSS/BETA) + UPSIPI*log(pinff/PINFSS) + UPSIY*log(yf/yf));
// 49 Balance sheet policy
    bal2yf = RESS/YSS;
// 50 Consumption Euler equation
    1 = lamnf(+1)*rf;
// 51 Labor supply
    whf = PSI*hf^ETA*cf^GAMMA;
// 52 Nominal SDF
    lamnf = BETA*epsn/epsn(-1)*(cf(-1)/cf)^GAMMA/pinff;
// 53 Real SDF
    lamf = lamnf*pinff;
// 54 Reset wage
    wstarf = (1+SW)*EPSW/(EPSW-1)*whf;
// 57 Labor market clearing
    hf = hdf*vwf;
// 58 Wage dispersion
    vwf = (wstarf/wf)^(-EPSW);
// 59 Aggregate wage
    wf^(1-EPSW)=wstarf^(1-EPSW);
// 60 Labor demand
    wf = (1+SP)*mcf*(1-ALPHA)*ywf/hdf;
// 61 Capital return
    pkf*xif = lamf(+1)*((1+SK)*mcf(+1)*ALPHA*ywf(+1)/kf+(1-DELTA)*pkf(+1)*xif(+1));
// 62 Optimal investment
     1 = pkf*epsi*(1 - PHII*(invf/invf(-1)-1)*invf/invf(-1) - PHII/2*(invf/invf(-1)-1)^2)
        + lamf(+1)*xif(+1)/xif*pkf(+1)*epsi(+1)*PHII*(invf(+1)/invf-1)*(invf(+1)/invf)^2;
// 63 Bond pricing
    qf*xif = lamnf(+1)*(1+KAPPA*qf(+1)*xif(+1));
// 64 Capital accumulation equation
    kf = (1-DELTA)*kf(-1) + epsi*(1-PHII/2*(invf/invf(-1)-1)^2)*invf;
// 65 Loan-in-advance constraint
    invf = qf*(ff-KAPPA*ff(-1)/pinff);
// 66 Reset price
    pstarf = EPSP/(EPSP-1)*mcf;
// 69 Aggregate production
    ywf = epsa*kf(-1)^ALPHA*hdf^(1-ALPHA);
// 70 Goods market clearing
    ywf = yf*vpf;
// 71 Price dispersion
    vpf = pstarf^(-EPSP);
// 72 Inflation
    1 = pstarf^(1-EPSP);
// 73 Long-term rate
    rlf = (1+KAPPA*qf)/qf(-1);
// 74 Equity adjustment
    1+PHIN*nf/NSS*(nf/NSS-1)+PHIN/2*(nf/NSS-1)^2 = ZETA*lamnf(+1)*(rf+(rlf(+1)-rf)*phif);
// 75 Risk-weighted leverage constraint
    epsf*phif*lamnf(+1)*rlf(+1) = lamnf(+1)*(rf+(rlf(+1)-rf)*phif);
// 76 Risk-weighted leverage
    phif*nf = qf*(ff+BG)-ref;
// 77 Central bank balance sheet
    qf*bcbf = ref;
// 78 Debt-market clearing
    bf = BG - bcbf;
// 79 Aggregate resource constraint
    yf = cf + invf;
// 80 Relative balance sheet
    bal2yf = ref/yf;
end;

options_.noprint=1;
steady;
check;

shocks;
var ea = 1;
var en = 1;
var ef = 1;
var ei = 1;
end;

stoch_simul(order=2,irf=0,nomoments);

base_paramfile
W = welfhh + 0.5*oo_.dr.ghs2(oo_.dr.inv_order_var(1));
lambda = 100*(1 - (((1-BETA)*W + PSI*h^(1+ETA)/(1+ETA))*(1-GAMMA))^(1/(1-GAMMA))/c);

stoch_simul(order=1,irf=21,nograph,qz_zero_threshold=1e-8);