warning off

// Endogenous variables
var welfhh  // household welfare
    Util1   // utility
    Util2   // utility
    y       // aggregate output
    yw      // wholesale output
    c       // consumption
    k       // physical capital stock
    inv     // investment
    h       // labor supply
    hd      // labor demand
    f       // investment bonds
    n       // bank equity
    phi     // risk-weighted leverage
    b       // bank treasuries
    bcb     // central bank treasuries
    re      // reserves
    lamn    // nominal sdf
    lam     // sdf
    r       // policy rate
    mc      // marginal cost
    pinf    // inflation
    pstar   // optimal reset price
    x1      // price numerator
    x2      // price denominator
    vp      // price dispersion
    wh      // household wage
    w       // aggregate wage
    wstar   // reset wage
    g1      // wage numerator
    g2      // wage denominator
    vw      // wage dispersion
    pk      // physical capital price
    xi      // liquidity premium
    q       // bond price
    rl      // long-term rate
    rlong   // long-term yield
    qeh     // bond price (expectations hypothesis)
    rlongeh // long-term yield (expectations hypothesis)
    tp      // term premium
    real    // ex ante real rate
    epinf   // expected inflation
    bal2y   // relative balance sheet size
    bal2yl; // lag relative balance sheet size

// Exogenous variables
var epsa    // technology
    epsf    // financial
    epsi    // marginal efficiency of investment
    epsn;   // preference

varexo ea en ef ei er eb;

parameters  BETA ZETA GAMMA PSI ETA ALPHA DELTA KAPPA EPSP EPSW PINFSS RESS
            PHIP PHIW PHII PHIN RHOR UPSIPI UPSIY NSS BG
            RHOA SIGA RHON SIGP RHOF SIGF EPSF RHOI SIGI SIGR YSS
            RHORE1 RHORE2 SIGRE SW SP SK;

model;
// OPTIMAL MONETARY POLICY
// Policy Rule, agent 1
    log(r) = RHOR*log(r(-1)) + (1-RHOR)*(log(PINFSS/BETA) + UPSIPI*log(pinf/PINFSS)) - SIGR*er;
// Policy Rule, agent 2
    bal2y = (1-RHORE1-RHORE2)*RESS/YSS + RHORE1*bal2yl + RHORE2*bal2yl(-1) + SIGRE*eb;
// 3  Utility 1
    Util1 = c^(1-GAMMA)/(1-GAMMA) - PSI*h^(1+ETA)/(1+ETA);
// 4  Utility 2
    Util2 = c^(1-GAMMA)/(1-GAMMA) - PSI*h^(1+ETA)/(1+ETA);

// MODEL
// 5  Ex ante real rate
    real = r/pinf(+1);
// 6  Consumption Euler equation
    1 = lamn(+1)*r;
// 7  Labor supply
    wh = PSI*h^ETA*c^GAMMA;
// 8  Nominal SDF
    lamn = BETA*epsn/epsn(-1)*(c(-1)/c)^GAMMA/pinf;
// 9  Real SDF
    lam = lamn*pinf;
// 10 Reset wage
    wstar = (1+SW)*EPSW/(EPSW-1)*g1/g2;
// 11 Wage numerator
    g1 = wh*w^EPSW*hd+PHIW*lam(+1)*pinf(+1)^EPSW*g1(+1);
// 12 Wage denominator
    g2 = w^EPSW*hd+PHIW*lam(+1)*pinf(+1)^(EPSW-1)*g2(+1);
// 13 Labor market clearing
    h = hd*vw;
// 14 Wage dispersion
    vw = (1-PHIW)*(wstar/w)^(-EPSW)+PHIW*pinf^EPSW*(w/w(-1))^EPSW*vw(-1);
// 15 Aggregate wage
    w^(1-EPSW)=(1-PHIW)*wstar^(1-EPSW)+PHIW*pinf^(EPSW-1)*w(-1)^(1-EPSW);
// 16 Labor demand
    w = (1+SP)*mc*(1-ALPHA)*yw/hd;
// 17 Capital return
    pk*xi = lam(+1)*((1+SK)*mc(+1)*ALPHA*yw(+1)/k+(1-DELTA)*pk(+1)*xi(+1));
// 18 Optimal investment
     1 = pk*epsi*(1 - PHII*(inv/inv(-1)-1)*inv/inv(-1) - PHII/2*(inv/inv(-1)-1)^2)
        + lam(+1)*xi(+1)/xi*pk(+1)*epsi(+1)*PHII*(inv(+1)/inv-1)*(inv(+1)/inv)^2;
// 19 Bond pricing
    q*xi = lamn(+1)*(1+KAPPA*q(+1)*xi(+1));
// 20 Capital accumulation equation
    k = (1-DELTA)*k(-1) + epsi*(1-PHII/2*(inv/inv(-1)-1)^2)*inv;
// 21 Loan-in-advance constraint
    inv = q*(f-KAPPA*f(-1)/pinf);
// 22 Reset price
    pstar = EPSP/(EPSP-1)*x1/x2;
// 23 Price numerator
    x1 = mc*y+PHIP*lam(+1)*pinf(+1)^EPSP*x1(+1);
// 24 Price denominator
    x2 = y+PHIP*lam(+1)*pinf(+1)^(EPSP-1)*x2(+1);
// 25 Aggregate production
    yw = epsa*k(-1)^ALPHA*hd^(1-ALPHA);
// 26 Goods market clearing
    yw = y*vp;
// 27 Price dispersion
    vp = (1-PHIP)*pstar^(-EPSP)+PHIP*pinf^EPSP*vp(-1);
// 28 Inflation
    1 = (1-PHIP)*pstar^(1-EPSP)+PHIP*pinf^(EPSP-1);
// 29 Long-term rate
    rl = (1+KAPPA*q)/q(-1);
// 30 Equity adjustment
    1+PHIN*n/NSS*(n/NSS-1)+PHIN/2*(n/NSS-1)^2 = ZETA*lamn(+1)*(r+(rl(+1)-r)*phi);
// 31 Risk-weighted leverage constraint
    epsf*phi*lamn(+1)*rl(+1) = lamn(+1)*(r+(rl(+1)-r)*phi);
// 32 Risk-weighted leverage
    phi*n = q*(f+b);
// 33 Central bank balance sheet
    q*bcb = re;
// 34 Debt-market clearing
    b = BG - bcb;
// 35 Aggregate resource constraint
    y = c + inv + PHIN/2*(n/NSS-1)^2;
// 36 Long-term yield
    rlong = 1/q+KAPPA;
// 37 Expectations hypothesis bond price
    qeh = (1+KAPPA*qeh(+1))/r;
// 38 Expectations hypothesis long-term yield
    rlongeh = 1/qeh+KAPPA;
// 39 Term premium
    tp = rlong - rlongeh;

// EXOGENOUS PROCESSES
// 40 Technology
    log(epsa) = RHOA*log(epsa(-1)) + SIGA*ea;
// 41 Preferences
    log(epsn) = RHON*log(epsn(-1)) - SIGP*en;
// 42 Financial
    log(epsf) = (1-RHOF)*log(EPSF) + RHOF*log(epsf(-1)) - SIGF*ef;
// 43 Marginal efficiency of investment
    log(epsi) = RHOI*log(epsi(-1)) + SIGI*ei;

// WELFARE MEASURE
// 44 Household welfare
    welfhh = Util1 + BETA*welfhh(+1);

// 45 Expected inflation
    epinf = pinf(+1);
// 46 Relative size of the balance sheet
    bal2y = re/y;
// 47 Lag relative size of the balance sheet
    bal2yl = bal2y(-1);
end;

options_.noprint=1;
steady;
check;

shocks;
var ea = 1;
var en = 1;
var ef = 1;
var ei = 1;
var er = 1;
var eb = 1;
end;

// Calibration
//stoch_simul(order=1,irf=21,nograph);
//decomp = oo_.gamma_y{options_.ar+2}(4,:);
//stoch_simul(order=2,periods=5000,irf=0,pruning,nograph);
//stdyprem = [std(log(oo_.endo_simul(4,options_.drop+1:end))) std(log(oo_.endo_simul(6,options_.drop+1:end))) std(oo_.endo_simul(50,options_.drop+1:end))];

// Actual computations
stoch_simul(order=2,irf=21,nograph);