clc;
clear all;

addpath('d:\dynare\4.4.3\matlab')

EPSW = 6;
EPSP = 6;
PHIW = 0.75;
PHIP = 0.75;
SPREAD = 135;
DELTA = 0.025;
ALPHA = 1/3;
DURATION = 40;
TREAS2DEBT = 0.4;
ETA = 2;
GAMMA = 2;

pinf = 1;
h = 1/3;
phi = 6;
r = 1.02^0.25;
re = 0;
            
PHII = 2;
PHIN = 0.4295;
RHOR = 0.8;
UPSIPI = 2;
UPSIY = 0.0;

load ./estimation/base_new_mean.mat
for it = 1:size(parameter_names,1)
    eval([parameter_names{it} '= xparam1(' num2str(it) ');'])
end

RHORE1 =  0.950712191157027; % 1.021132476011252;
RHORE2 = 0; % -0.217764317642134;
SIGRE = 0.708382404601087;

save params EPSW EPSP PHIW PHIP SPREAD DELTA ALPHA DURATION TREAS2DEBT ETA ...
            GAMMA pinf h phi r  re PHII PHIN RHOR UPSIPI UPSIY ...
            RHOA RHON RHOF RHOI SIGA SIGP SIGF SIGI SIGR SIGRE ...
            RHORE1 RHORE2
       
dynare base.mod noclearall

SIGRE = fsolve(@(x) calib_bal(x),SIGRE)

save params EPSW EPSP PHIW PHIP SPREAD DELTA ALPHA DURATION TREAS2DEBT ETA ...
            GAMMA pinf h phi r  re PHII PHIN RHOR UPSIPI UPSIY ...
            RHOA RHON RHOF RHOI SIGA SIGP SIGF SIGI SIGR SIGRE ...
            RHORE1 RHORE2
        
RHORE1 = fsolve(@(x) calib_bal2(x),RHORE1)

% save params EPSW EPSP PHIW PHIP SPREAD DELTA ALPHA DURATION TREAS2DEBT ETA ...
%             GAMMA pinf h phi r  re PHII PHIN RHOR UPSIPI UPSIY ...
%             RHOA RHON RHOF RHOI SIGA SIGP SIGF SIGI SIGR SIGRE ...
%             RHORE1 RHORE2
% 
% RHORE2 = fsolve(@(x) calib_bal3(x),RHORE2)

save params EPSW EPSP PHIW PHIP SPREAD DELTA ALPHA DURATION TREAS2DEBT ETA ...
            GAMMA pinf h phi r  re PHII PHIN RHOR UPSIPI UPSIY ...
            RHOA RHON RHOF RHOI SIGA SIGP SIGF SIGI SIGR SIGRE ...
            RHORE1 RHORE2
base
base_paramfile
base_ss_defs

shocks = {'ea','en','ef','ei','er','eb'};
log_vars = {'welfhh','Util1','Util2','y','yw','c','k','inv','h','f','n','phi','b','bcb','re','mc','wh','w','wstar','pk','xi','q','qeh','epsa','epsf','epsi','epsn'}; 
level_vars = {'bal2y'}; 
level_ann_vars = {'r','pinf','real','rl','rlong','rlongeh','tp','epinf'}; 

for it = 1:size(shocks,2)
    for it2 = 1:size(log_vars,2)
        eval(['irf.base.' log_vars{it2} '_' shocks{it} '= 100*' log_vars{it2} '_' shocks{it} '/' log_vars{it2} ';']);
    end
    for it2 = 1:size(level_vars,2)
        eval(['irf.base.' level_vars{it2} '_' shocks{it} '= 100*' level_vars{it2} '_' shocks{it} ';']);
    end
    for it2 = 1:size(level_ann_vars,2)
        eval(['irf.base.' level_ann_vars{it2} '_' shocks{it} '= 400*' level_ann_vars{it2} '_' shocks{it} ';']);
    end
end

varlist = {'y','c','h','inv','rlong','tp','n','phi','r','epinf','bal2y'};
varnames = {'Output','Consumption','Labor','Investment',...
            '10-Year Yield','Term Premium','Bank Equity',...
            'Risk-weighted Leverage','Policy Rate','Expected Inflation',...
            'CB Balance Sheet-to-GDP'};

% FIGURE 1
figure('units','normalized','outerposition',[0 0 1 1])
for itv = 1:size(varlist,2)
    subplot(3,4,itv)
    eval(['plot(0:20,irf.base.' varlist{itv} '_er,''color'',[0 32 91]/255,''linewidth'',4)'])
    hold on;
    eval(['plot(0:20,irf.base.' varlist{itv} '_eb,''-.'',''color'',[0 107 166]/255,''linewidth'',4)'])
    plot(0:20,zeros(size(0:20)),'k','linewidth',2); 
    hold off;
    set(gca,'fontname', 'times','fontsize',24,'YGrid', 'on', 'XGrid', 'off');
    eval(['title(''' varnames{itv} ''',''fontname'',''times'',''fontsize'',30,''FontWeight'',''Normal'')'])
end
subplot(3,4,12)
plot(0:20,100*ones(size(0:20)),'color',[0 32 91]/255,'linewidth',4)
hold on;
plot(0:20,100*ones(size(0:20)),'-.','color',[0 107 166]/255,'linewidth',4)
plot(0:20,zeros(size(0:20)),'k','linewidth',2); 
hold off;
ylim([-1 1])
legend({'Policy Rate','Balance Sheet'},'fontname','times','fontsize',24,'location','best')
legend boxoff	
set(gca,'fontname', 'times','fontsize',24,'YGrid', 'on', 'XGrid', 'off');
title('Policy Shock','fontname','times','fontsize',30,'FontWeight','Normal')
% print ../figures_replic/irf_policy -depsc
close all

%% clean-up folder
delete('*.mat')
delete('*.log')
delete('*.asv')
delete('*.jnl')
delete('*_static.m')
delete('*_dynamic.m')
delete('*_auxiliary_variables.m')

delete('base.m')
rmdir('base','s')